import SimpleGUICS2Pygame.simpleguics2pygame as simplegui
import math
import random

# Размеры холста
WIDTH = 400
HEIGHT = 400

# Характеристики мячика.
RADIUS = 20
COLOR = "White"

# Скалярное произведение (вспомогательная математическая функция).
def dot(v, w):
	return v[0] * w[0] + v[1] * w[1]
	
class RectangularDomain:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.border = 2
	
    # Определяет, находится ли мячик внутри домена.
    def inside(self, center, radius):
        return ((center[0] - radius > self.border ) and
                (center[1] - radius > self.border ) and
                (center[0] + radius < self.width - self.border) and
                (center[1] + radius < self.height - self.border))

    # Возвращает единичную нормаль к границе области в ближайшей к center точке.
    def normal(self, center):
        dx1 = center[0] - self.border
        dx2 = self.width - self.border - center[0]
        dy1 = center[1] - self.border
        dy2 = self.height - self.border - center[1]
        min_d = min(dx1, dx2, dy1, dy2)

        normal = [0, 0]

        if dx1 == min_d:
            normal[0] += 1
        elif dy1 == min_d:
            normal[1] += 1
        elif dx2 == min_d:
            normal[0] += -1
        elif dy2 == min_d:
            normal[1] += -1

        return normal
	
    # Возвращает случайное положение внутри домена.
    def random_pos(self, radius):
        x = random.randrange(self.border + radius,self.width - self.border - radius)
        y = random.randrange(self.border + radius,self.height - self.border - radius)
        return [x, y]
	
    # Рисует границы домена.
    def draw(self, canvas):
        canvas.draw_polygon([[0, 0], [self.width - self.border, 0],
                             [self.width - self.border, self.height - self.border],
                             [0, self.height - self.border]], self.border, "red")


# Круглая область.
class CircularDomain:
    def __init__(self, center, radius):
        self.center = center
        self.radius = radius
        self.border = 2

    # Определяет, находится ли мячик внутри домена.
    def inside(self,center,radius):
        dx = center[0] - self.center[0]
        dy = center[1] - self.center[1]
        dr = math.sqrt(dx ** 2 + dy ** 2)
        return dr < (self.radius - radius - self.border)

    # Возвращает единичную нормаль к границе области в ближайшей к center точке.
    def normal(self, center):
        dx = center[0] - self.center[0]
        dy = center[1] - self.center[1]
        dr = math.sqrt(dx ** 2 + dy ** 2)
        return [dx / dr, dy / dr]

    # Возвращает случайное положение внутри домена.
    def random_pos(self, radius):
        r=random.random() * (self.radius - radius - self.border)
        theta = random.random() * 2 * math.pi
        x = r * math.cos(theta) + self.center[0]
        y = r * math.sin(theta) + self.center[1]
        return [x, y]

    # Рисует границы домена.
    def draw(self,canvas):
        canvas.draw_circle(self.center,self.radius,self.border,"green")

# Мячик.
class Ball:
	def __init__(self, radius, color, domain):
		self.radius = radius
		self.color = color
		self.domain = domain
		
		self.pos = self.domain.random_pos(self.radius)
		self.vel = [3 * random.random() + .1, 3 * random.random() + .1]
		
	# Отражение.
	def reflect(self):
		norm = self.domain.normal(self.pos)
		norm_length = dot(self.vel, norm)
		self.vel[0] = self.vel[0] - 2 * norm_length * norm[0]
		self.vel[1] = self.vel[1] - 2 * norm_length * norm[1]
		
	# Обновление позиции мячика.
	def update(self):
		self.pos[0] += self.vel[0]
		self.pos[1] += self.vel[1]
		if not self.domain.inside(self.pos, self.radius):
			self.reflect()
			
	# Рисование.
	def draw(self, canvas):
		canvas.draw_circle(self.pos, self.radius, 1, self.color, self.color)

balls = [ ]
		
# Обновление позиции шарика и рисование.
def draw(canvas):
        for ball in balls:
            ball.update()
        field.draw(canvas)
        for ball in balls:
            ball.draw(canvas)
	
field = RectangularDomain(WIDTH, HEIGHT)
#field = CircularDomain([WIDTH / 2, HEIGHT / 2], 180)

balls.append(Ball(RADIUS, COLOR, field))
balls.append(Ball(RADIUS, 'Green', field))

frame = simplegui.create_frame("Передвижения шарика", WIDTH, HEIGHT)

frame.set_draw_handler(draw)

frame.start()
